@@ -7,13 +7,9 @@ package one.heatmap;
 
 import java.io.IOException;
 import java.io.PrintStream;
-import java.util.Arrays;
-import java.util.Comparator;
+import java.util.*;
 
-import one.convert.Arguments;
-import one.convert.Index;
-import one.convert.JfrConverter;
-import one.convert.ResourceProcessor;
+import one.convert.*;
 import one.jfr.DictionaryInt;
 
 public class Heatmap {
@@ -28,11 +24,11 @@ public class Heatmap {
 
     public Heatmap(Arguments args, JfrConverter converter) {
         this.args = args;
-        this.state = new State(converter, BLOCK_DURATION_MS);
+        this.state = new State(converter, args, BLOCK_DURATION_MS);
     }
 
-    public void addEvent(int stackTraceId, int extra, byte type, long timeMs) {
-        state.addEvent(stackTraceId, extra, type, timeMs);
+    public void addEvent(int stackTraceId, int threadId, int classId, byte type, long timeMs) {
+        state.addEvent(stackTraceId, threadId, classId, type, timeMs);
     }
 
     public void addStack(long id, long[] methods, int[] locations, byte[] types, int size) {
@@ -40,12 +36,12 @@ public class Heatmap {
     }
 
     public void beforeChunk() {
-        state.methodsCache.clear();
+        state.methodCache.clear();
     }
 
     public void finish(long startMs) {
         this.startMs = startMs;
-        state.methodsCache.clear();
+        state.methodCache.clear();
         state.stackTracesCache.clear();
     }
 
@@ -54,9 +50,9 @@ public class Heatmap {
         this.state = null;
         return new EvaluationContext(
                 state.sampleList.samples(),
-                state.methodsCache.methodsIndex(),
+                state.methods,
                 state.stackTracesRemap.orderedTraces(),
-                state.methodsCache.orderedSymbolTable()
+                state.symbolTable.keys()
         );
     }
 
@@ -194,7 +190,7 @@ public class Heatmap {
             int methodId = context.nodeTree.extractMethodId(d);
 
             out.writeVar(synonymTable.nodeIdOrSynonym(parentId));
-            out.writeVar(context.orderedMethods[methodId].frequencyOrNewMethodId);
+            out.writeVar(context.orderedMethods[methodId].frequencyBasedId);
         }
     }
 
@@ -215,26 +211,24 @@ public class Heatmap {
         out.writeVar(startsCount);
         for (Method method : context.orderedMethods) {
             if (method.start) {
-                out.writeVar(method.frequencyOrNewMethodId);
+                out.writeVar(method.frequencyBasedId);
             }
         }
     }
 
     private void renameMethodsByFrequency(EvaluationContext context) {
-        Arrays.sort(context.orderedMethods, new Comparator<Method>() {
+        Method[] methodsByFrequency = context.orderedMethods.clone();
+        Arrays.sort(methodsByFrequency, new Comparator<Method>() {
             @Override
             public int compare(Method o1, Method o2) {
-                return Integer.compare(o2.frequencyOrNewMethodId, o1.frequencyOrNewMethodId);
+                return Integer.compare(o2.frequency, o1.frequency);
             }
         });
 
-        for (int i = 0; i < context.orderedMethods.length; i++) {
-            Method method = context.orderedMethods[i];
-            method.frequencyOrNewMethodId = i + 1; // zero is reserved for no method
+        for (int i = 0; i < methodsByFrequency.length; i++) {
+            Method method = methodsByFrequency[i];
+            method.frequencyBasedId = i + 1; // zero is reserved for no method
         }
-
-        // restores order
-        context.methods.keys(context.orderedMethods);
     }
 
     private int[] buildLz78TreeAndPrepareData(EvaluationContext context) {
@@ -263,7 +257,7 @@ public class Heatmap {
                 for (int methodId : stack) {
                     current = context.nodeTree.appendChild(current, methodId);
                     if (current == 0) { // so we are starting from root again, it will be written to output as Lz78 element - [parent node id; method id]
-                        context.orderedMethods[methodId].frequencyOrNewMethodId++;
+                        context.orderedMethods[methodId].frequency++;
                         if (stackBuffer.length == chunksIterator) {
                             stackBuffer = Arrays.copyOf(stackBuffer, chunksIterator + chunksIterator / 2);
                         }
@@ -288,7 +282,7 @@ public class Heatmap {
                 for (int methodId : stack) {
                     current = context.nodeTree.appendChild(current, methodId);
                     if (current == 0) { // so we are starting from root again, it will be written to output as Lz78 element - [parent node id; method id]
-                        context.orderedMethods[methodId].frequencyOrNewMethodId++;
+                        context.orderedMethods[methodId].frequency++;
                     }
                 }
             }
@@ -351,7 +345,7 @@ public class Heatmap {
         Arrays.sort(evaluationContext.orderedMethods, new Comparator<Method>() {
             @Override
             public int compare(Method o1, Method o2) {
-                return Integer.compare(o1.frequencyOrNewMethodId, o2.frequencyOrNewMethodId);
+                return Integer.compare(o1.frequencyBasedId, o2.frequencyBasedId);
             }
         });
         out.nextByte('A');
@@ -372,7 +366,6 @@ public class Heatmap {
     }
 
     private static class EvaluationContext {
-        final Index<Method> methods;
         final Method[] orderedMethods;
         final int[][] stackTraces;
         final String[] symbols;
@@ -383,10 +376,8 @@ public class Heatmap {
 
         EvaluationContext(SampleList.Result sampleList, Index<Method> methods, int[][] stackTraces, String[] symbols) {
             this.sampleList = sampleList;
-            this.methods = methods;
             this.stackTraces = stackTraces;
             this.symbols = symbols;
-
             orderedMethods = methods.keys();
         }
     }
@@ -395,48 +386,61 @@ public class Heatmap {
 
         private static final int LIMIT = Integer.MAX_VALUE;
 
+        final JfrConverter converter;
+        final Arguments args;
         final SampleList sampleList;
         final StackStorage stackTracesRemap = new StackStorage();
 
+        // Maps stack trace ID to prototype ID in stackTracesRemap
         final DictionaryInt stackTracesCache = new DictionaryInt();
-        final MethodCache methodsCache;
+        final Map<MethodKey, Integer> methodCache = new HashMap<>();
+        final Index<Method> methods = new Index<>(Method.class, Method.EMPTY);
+        final Index<String> symbolTable = new Index<>(String.class, "");
 
         // reusable array to (temporary) store (potentially) new stack trace
         int[] cachedStackTrace = new int[4096];
 
-        State(JfrConverter converter, long blockDurationMs) {
-            sampleList = new SampleList(blockDurationMs);
-            methodsCache = new MethodCache(converter);
+        State(JfrConverter converter, Arguments args, long blockDurationMs) {
+            this.converter = converter;
+            this.args = args;
+            this.sampleList = new SampleList(blockDurationMs);
         }
 
-        public void addEvent(int stackTraceId, int extra, byte type, long timeMs) {
+        public void addEvent(int stackTraceId, int threadId, int classId, byte type, long timeMs) {
             if (sampleList.getRecordsCount() >= LIMIT) {
                 return;
             }
-            if (extra == 0) {
-                sampleList.add(stackTracesCache.get(stackTraceId), timeMs);
-                return;
-            }
-
-            int id = stackTracesCache.get((long) extra << 32 | stackTraceId, -1);
-            if (id != -1) {
-                sampleList.add(id, timeMs);
-                return;
-            }
 
             int prototypeId = stackTracesCache.get(stackTraceId);
+            if (classId == 0 && !args.threads) {
+                sampleList.add(prototypeId, timeMs);
+                return;
+            }
+
             int[] prototype = stackTracesRemap.get(prototypeId);
+            int stackSize = prototype.length + (args.threads ? 1 : 0) + (classId != 0 ? 1 : 0);
+            if (cachedStackTrace.length < stackSize) {
+                cachedStackTrace = new int[stackSize * 2];
+            }
 
-            id = stackTracesRemap.indexWithPrototype(prototype, methodsCache.indexForClass(extra, type));
-            stackTracesCache.put((long) extra << 32 | stackTraceId, id);
+            if (args.threads) {
+                MethodKey key = new MethodKey(MethodKeyType.THREAD, threadId, -1, Frame.TYPE_NATIVE, true);
+                cachedStackTrace[0] = getMethodIndex(key);
+            }
 
-            sampleList.add(id, timeMs);
+            System.arraycopy(prototype, 0, cachedStackTrace, args.threads ? 1 : 0, prototype.length);
+
+            if (classId != 0) {
+                MethodKey key = new MethodKey(MethodKeyType.CLASS, classId, -1, type, false);
+                cachedStackTrace[stackSize - 1] = getMethodIndex(key);
+            }
+
+            sampleList.add(stackTracesRemap.index(cachedStackTrace, stackSize), timeMs);
         }
 
         public void addStack(long id, long[] methods, int[] locations, byte[] types, int size) {
-            int[] stackTrace = cachedStackTrace;
-            if (stackTrace.length < size) {
-                cachedStackTrace = stackTrace = new int[size * 2];
+            if (cachedStackTrace.length < size) {
+                cachedStackTrace = new int[size * 2];
             }
 
             for (int i = size - 1; i >= 0; i--) {
@@ -447,12 +451,89 @@ public class Heatmap {
                 int index = size - 1 - i;
                 boolean firstMethodInTrace = index == 0;
 
-                stackTrace[index] = methodsCache.index(methodId, location, type, firstMethodInTrace);
+                // When args.threads is true, the first frame is the artificial thread frame
+                boolean firstFrameInStack = firstMethodInTrace && !args.threads;
+
+                MethodKey key = new MethodKey(MethodKeyType.METHOD, methodId, location, type, firstFrameInStack);
+                cachedStackTrace[index] = getMethodIndex(key);
             }
 
-            stackTracesCache.put(id, stackTracesRemap.index(stackTrace, size));
+            stackTracesCache.put(id, stackTracesRemap.index(cachedStackTrace, size));
         }
 
+        private int getMethodIndex(MethodKey key) {
+            Integer oldIdx = methodCache.get(key);
+            if (oldIdx != null) return oldIdx;
+
+            int newIdx = methods.index(key.makeMethod(converter, symbolTable));
+            methodCache.put(key, newIdx);
+            return newIdx;
+        }
+
+        private static final class MethodKey {
+            private final long methodId;
+            // 32 bits: location
+            // 8 bits: type
+            // 1 bit: firstInStack
+            private final long metadata;
+            // Used to infer what type of method to create
+            private final MethodKeyType keyType;
+
+            public MethodKey(MethodKeyType keyType, long methodId, int location, byte type, boolean firstInStack) {
+                this.keyType = keyType;
+                this.methodId = methodId;
+                this.metadata = (long) (firstInStack ? 1 : 0) << 40 | (type & 0xffL) << 32 | (location & 0xFFFFFFFFL);
+            }
+
+            public int getLocation() {
+                return (int) metadata;
+            }
+
+            public byte getType() {
+                return (byte) (metadata >> 32);
+            }
+
+            public boolean getFirstInStack() {
+                return ((metadata >> 40) & 1L) != 0;
+            }
+
+            public Method makeMethod(JfrConverter converter, Index<String> symbolTable) {
+                switch (keyType) {
+                    case METHOD:
+                        StackTraceElement ste = converter.getStackTraceElement(methodId, getType(), getLocation());
+                        int className = symbolTable.index(ste.getClassName());
+                        int methodName = symbolTable.index(ste.getMethodName());
+                        return new Method(className, methodName, getLocation(), getType(), getFirstInStack());
+
+                    case THREAD:
+                        String threadName = converter.getThreadName(Math.toIntExact(methodId));
+                        return new Method(0, symbolTable.index(threadName), getLocation(), getType(), getFirstInStack());
+
+                    case CLASS:
+                        String javaClassName = converter.getClassName(methodId);
+                        return new Method(symbolTable.index(javaClassName), 0, getLocation(), getType(), getFirstInStack());
+
+                    default:
+                        throw new IllegalArgumentException("Unexpected keyType: " + keyType);
+                }
+            }
+
+            @Override
+            public boolean equals(Object other) {
+                if (!(other instanceof MethodKey)) return false;
+                MethodKey methodKey = (MethodKey) other;
+                return methodId == methodKey.methodId && metadata == methodKey.metadata && keyType == methodKey.keyType;
+            }
+
+            @Override
+            public int hashCode() {
+                return 31 * (31 * Long.hashCode(methodId) + Long.hashCode(metadata)) + keyType.hashCode();
+            }
+        }
+
+        private enum MethodKeyType {
+            METHOD, THREAD, CLASS
+        }
     }
 
 }
