@@ -12,17 +12,26 @@ import java.util.HashMap;
 
 public class MallocLeakAggregator implements EventCollector {
     private final EventCollector wrapped;
+    private final double tail;
     private final Map<Long, MallocEvent> addresses;
     private List<MallocEvent> events;
+    private long minTime = Long.MAX_VALUE;
+    private long maxTime = Long.MIN_VALUE;
 
-    public MallocLeakAggregator(EventCollector wrapped) {
+    public MallocLeakAggregator(EventCollector wrapped, double tail) {
+        if (tail < 0.0 || tail > 1.0) {
+            throw new IllegalArgumentException("tail must be between 0 and 1");
+        }
         this.wrapped = wrapped;
+        this.tail = tail;
         this.addresses = new HashMap<>();
     }
 
     @Override
     public void collect(Event e) {
         events.add((MallocEvent) e);
+        minTime = Math.min(minTime, e.time);
+        maxTime = Math.max(maxTime, e.time);
     }
 
     @Override
@@ -47,9 +56,15 @@ public class MallocLeakAggregator implements EventCollector {
 
     @Override
     public boolean finish() {
+        // Ignore tail allocations made in the last N% of profiling session:
+        // they are too young to be considered a leak
+        long timeCutoff = (long) (minTime * tail + maxTime * (1.0 - tail));
+
         wrapped.beforeChunk();
         for (Event e : addresses.values()) {
-            wrapped.collect(e);
+            if (e.time <= timeCutoff) {
+                wrapped.collect(e);
+            }
         }
         wrapped.afterChunk();
 
