@@ -58,13 +58,15 @@ enum JfrType {
     T_INITIAL_SYSTEM_PROPERTY = 112,
     T_NATIVE_LIBRARY = 113,
     T_GC_HEAP_SUMMARY = 114,
-    T_LOG = 115,
-    T_WINDOW = 116,
-    T_LIVE_OBJECT = 117,
-    T_WALL_CLOCK_SAMPLE = 118,
-    T_MALLOC = 119,
-    T_FREE = 120,
-    T_USER_EVENT = 121,
+    T_METHOD_TRACE = 115,
+    T_LOG = 116,
+    T_WINDOW = 117,
+    T_LIVE_OBJECT = 118,
+    T_WALL_CLOCK_SAMPLE = 119,
+    T_MALLOC = 120,
+    T_FREE = 121,
+    T_USER_EVENT = 122,
+    T_PROCESS_SAMPLE = 123,
 
     // types after T_ANNOTATION inherit from java.lang.annotation.Annotation, see JfrMetadata::type
     T_ANNOTATION = 200,
