@@ -4,6 +4,7 @@
  */
 
 #include <dlfcn.h>
+#include <pthread.h>
 #include <stdlib.h>
 #include <string.h>
 #include "asprof.h"
@@ -18,7 +19,7 @@
 #ifdef __clang__
 #  define NO_OPTIMIZE __attribute__((optnone))
 #else
-#  define NO_OPTIMIZE __attribute__((optimize("O1")))
+#  define NO_OPTIMIZE __attribute__((optimize("-fno-omit-frame-pointer,-fno-optimize-sibling-calls")))
 #endif
 
 #define SAVE_IMPORT(FUNC) \
@@ -104,6 +105,32 @@ int MallocTracer::_patched_libs = 0;
 bool MallocTracer::_initialized = false;
 volatile bool MallocTracer::_running = false;
 
+static pthread_t _current_thread;
+static bool _nested_malloc = false;
+
+// Test if calloc() implementation calls malloc()
+static void* nested_malloc_hook(size_t size) {
+    if (pthread_self() == _current_thread) {
+        _nested_malloc = true;
+    }
+    return _orig_malloc(size);
+}
+
+// In some implementations, specifically on musl, calloc() calls malloc() internally,
+// and posix_memalign() calls aligned_alloc(). Detect such cases to prevent double-accounting.
+static void detectNestedMalloc() {
+    CodeCache* libc = Profiler::instance()->findLibraryByAddress((void*)_orig_calloc);
+    if (libc == NULL) {
+        return;
+    }
+
+    libc->patchImport(im_malloc, (void*)nested_malloc_hook);
+
+    _current_thread = pthread_self();
+    free(_orig_calloc(1, 1));
+    _current_thread = pthread_t(0);
+}
+
 // Call each intercepted function at least once to ensure
 // its GOT entry is updated with a correct target address
 static void resolveMallocSymbols() {
@@ -135,6 +162,8 @@ void MallocTracer::initialize() {
     SAVE_IMPORT(posix_memalign);
     SAVE_IMPORT(aligned_alloc);
 
+    detectNestedMalloc();
+
     lib->mark(
         [](const char* s) -> bool {
             return strcmp(s, "malloc_hook") == 0
@@ -170,8 +199,7 @@ void MallocTracer::patchLibraries() {
         cc->patchImport(im_free, (void*)free_hook);
         cc->patchImport(im_aligned_alloc, (void*)aligned_alloc_hook);
 
-        if (OS::isMusl()) {
-            // On musl, calloc() calls malloc() internally, and posix_memalign() calls aligned_alloc().
+        if (_nested_malloc) {
             // Use dummy hooks to prevent double-accounting. Dummy frames from AP are introduced
             // to preserve the frame link to the original caller (see #1226).
             cc->patchImport(im_calloc, (void*)calloc_hook_dummy);
