@@ -8,7 +8,6 @@
 
 #include <map>
 #include <string>
-#include <time.h>
 #include "arch.h"
 #include "arguments.h"
 #include "callTraceStorage.h"
@@ -60,7 +59,6 @@ class Profiler {
     std::map<int, std::string> _thread_names;
     std::map<int, jlong> _thread_ids;
     Dictionary _class_map;
-    Dictionary _symbol_map;
     ThreadFilter _thread_filter;
     CallTraceStorage _call_trace_storage;
     FlightRecorder _jfr;
@@ -68,8 +66,8 @@ class Profiler {
     Engine* _alloc_engine;
     int _event_mask;
 
-    time_t _start_time;
-    time_t _stop_time;
+    u64 _start_time;
+    u64 _stop_time;
     int _epoch;
     u32 _gc_id;
     WaitableMutex _timer_lock;
@@ -130,7 +128,7 @@ class Profiler {
     Engine* activeEngine();
     Error checkJvmCapabilities();
 
-    time_t addTimeout(time_t start, int timeout);
+    u64 addTimeout(u64 start_micros, int timeout);
     void startTimer();
     void stopTimer();
     void timerLoop(void* timer_id);
@@ -187,7 +185,7 @@ class Profiler {
     }
 
     u64 total_samples() { return _total_samples; }
-    long uptime()       { return time(NULL) - _start_time; }
+    long uptime()       { return (OS::micros() - _start_time) / 1000000ULL; }
 
     Dictionary* classMap() { return &_class_map; }
     ThreadFilter* threadFilter() { return &_thread_filter; }
@@ -225,7 +223,7 @@ class Profiler {
     bool isAddressInCode(const void* pc);
 
     void trapHandler(int signo, siginfo_t* siginfo, void* ucontext);
-    static void segvHandler(int signo, siginfo_t* siginfo, void* ucontext);
+    static void crashHandler(int signo, siginfo_t* siginfo, void* ucontext);
     static void wakeupHandler(int signo);
     static void setupSignalHandlers();
 
