@@ -7,6 +7,8 @@
 #define _STACKWALKER_H
 
 #include <stdint.h>
+#include "arguments.h"
+#include "event.h"
 #include "vmEntry.h"
 
 
@@ -25,23 +27,17 @@ struct StackContext {
     }
 };
 
-// Detail level of VMStructs stack walking
-enum StackDetail {
-    VM_BASIC,   // only basic Java frames similar to what AsyncGetCallTrace provides
-    VM_NORMAL,  // include frame types and runtime stubs
-    VM_EXPERT   // all features: frame types, runtime stubs, and intermediate native frames
-};
-
 class StackWalker {
   private:
     static int walkVM(void* ucontext, ASGCT_CallFrame* frames, int max_depth,
-                      StackDetail detail, const void* pc, uintptr_t sp, uintptr_t fp);
+                      StackWalkFeatures features, EventType event_type,
+                      const void* pc, uintptr_t sp, uintptr_t fp);
 
   public:
     static int walkFP(void* ucontext, const void** callchain, int max_depth, StackContext* java_ctx);
     static int walkDwarf(void* ucontext, const void** callchain, int max_depth, StackContext* java_ctx);
-    static int walkVM(void* ucontext, ASGCT_CallFrame* frames, int max_depth, StackDetail detail);
-    static int walkVM(void* ucontext, ASGCT_CallFrame* frames, int max_depth, JavaFrameAnchor* anchor);
+    static int walkVM(void* ucontext, ASGCT_CallFrame* frames, int max_depth, StackWalkFeatures features, EventType event_type);
+    static int walkVM(void* ucontext, ASGCT_CallFrame* frames, int max_depth, JavaFrameAnchor* anchor, EventType event_type);
 
     static void checkFault();
 };
