@@ -6,6 +6,18 @@
 #include <jni.h>
 #include <math.h>
 
+JNIEXPORT double largeInnerFrameFinal(int i) {
+    char frame[0x10000];
+    // Prevent from being optimized by compiler
+    (void)frame;
+
+    return sqrt(i) + pow(i, sqrt(i));
+}
+
+JNIEXPORT double largeInnerFrameIntermediate(int i) {
+    return largeInnerFrameFinal(i) + largeInnerFrameFinal(i + 1);
+}
+
 JNIEXPORT double doCpuTask() {
     int i = 0;
     double result = 0;
@@ -47,3 +59,11 @@ JNIEXPORT jdouble JNICALL Java_test_stackwalker_StackGenerator_deepFrame(JNIEnv*
 JNIEXPORT jdouble JNICALL Java_test_stackwalker_StackGenerator_leafFrame(JNIEnv* env, jclass cls) {
     return doCpuTask();
 }
+
+JNIEXPORT jdouble JNICALL Java_test_stackwalker_StackGenerator_largeInnerFrame(JNIEnv* env, jclass cls) {
+    double result = 0;
+    for (int i = 0; i < 100000000; i++) {
+        result += largeInnerFrameIntermediate(i);
+    }
+    return result;
+}
