@@ -179,6 +179,8 @@ public class Runner {
         TestDeclaration decl = TestDeclaration.parse(args);
         List<RunnableTest> allTests = decl.getRunnableTests();
         final int testCount = allTests.size();
+        final int retryCount = Integer.parseInt(System.getProperty("retryCount", "0"));
+
         int i = 1;
         long totalTestDuration = 0;
         List<String> failedTests = new ArrayList<>();
@@ -186,6 +188,14 @@ public class Runner {
         for (RunnableTest rt : allTests) {
             long start = System.nanoTime();
             TestResult result = run(rt, decl);
+
+            int attempt = 1;
+            while (result.status() == TestStatus.FAIL && attempt <= retryCount) {
+                log.log(Level.WARNING, "Test failed, retrying (attempt " + attempt + "/" + retryCount + ")...");
+                result = run(rt, decl);
+                attempt++;
+            }
+
             long durationNs = System.nanoTime() - start;
 
             totalTestDuration += durationNs;
