@@ -12,6 +12,7 @@ import one.profiler.test.Os;
 import one.profiler.test.Output;
 import one.profiler.test.Test;
 import one.profiler.test.TestProcess;
+import test.alloc.Hello;
 
 import java.time.Instant;
 import java.time.temporal.ChronoUnit;
@@ -19,18 +20,26 @@ import java.util.*;
 
 public class JfrTests {
 
-    @Test(mainClass = RegularPeak.class)
-    public void regularPeak(TestProcess p) throws Exception {
-        Output out = p.profile("-e cpu -d 6 -f %f.jfr");
-        String jfrOutPath = p.getFilePath("%f");
-        String peakPattern = "test/jfr/Cache.*calculateTop.*";
+    @Test(mainClass = CpuLoad.class, agentArgs = "start,event=cpu,file=%profile.jfr")
+    public void cpuLoad(TestProcess p) throws Exception {
+        p.waitForExit();
+        assert p.exitCode() == 0;
 
-        out = Output.convertJfrToCollapsed(jfrOutPath, "--to", "2500");
-        assert !out.contains(peakPattern);
-        out = Output.convertJfrToCollapsed(jfrOutPath,"--from", "2500", "--to", "5000");
-        assert out.samples(peakPattern) >= 1;
-        out = Output.convertJfrToCollapsed(jfrOutPath,"--from", "5000");
-        assert !out.contains(peakPattern);
+        String jfrOutPath = p.getFilePath("%profile");
+        String spikePattern = "test/jfr/CpuLoad.cpuSpike.*";
+        String normalLoadPattern = "test/jfr/CpuLoad.normalCpuLoad.*";
+
+        Output out = Output.convertJfrToCollapsed(jfrOutPath, "--to", "1500");
+        assert !out.contains(spikePattern);
+        assert out.contains(normalLoadPattern);
+
+        out = Output.convertJfrToCollapsed(jfrOutPath,"--from", "1500", "--to", "3500");
+        assert out.contains(spikePattern);
+        assert out.contains(normalLoadPattern);
+
+        out = Output.convertJfrToCollapsed(jfrOutPath,"--from", "3500");
+        assert !out.contains(spikePattern);
+        assert out.contains(normalLoadPattern);
     }
 
     /**
@@ -61,21 +70,29 @@ public class JfrTests {
      * @param p The test process to profile with.
      * @throws Exception Any exception thrown during profiling JFR output parsing.
      */
-    @Test(mainClass = JfrMultiModeProfiling.class, agentArgs = "start,event=cpu,alloc,lock,jfr,file=%f")
+    @Test(mainClass = JfrMultiModeProfiling.class, agentArgs = "start,event=cpu,alloc,lock=0,quiet,jfr,file=%f", output = true)
     public void parseMultiModeRecording(TestProcess p) throws Exception {
-        p.waitForExit();
+        Output output = p.waitForExit(TestProcess.STDOUT);
         assert p.exitCode() == 0;
+
+        long totalLockDurationMillis = output.stream().mapToLong(Long::parseLong).sum();
+
+        double jfrTotalLockDurationMillis = 0;
         Map<String, Integer> eventsCount = new HashMap<>();
         try (RecordingFile recordingFile = new RecordingFile(p.getFile("%f").toPath())) {
             while (recordingFile.hasMoreEvents()) {
                 RecordedEvent event = recordingFile.readEvent();
                 String eventName = event.getEventType().getName();
+                if (eventName.equals("jdk.JavaMonitorEnter")) {
+                    jfrTotalLockDurationMillis += event.getDuration().toNanos() / 1_000_000.0;
+                }
                 eventsCount.put(eventName, eventsCount.getOrDefault(eventName, 0) + 1);
             }
         }
 
         Assert.isGreater(eventsCount.get("jdk.ExecutionSample"), 50);
-        Assert.isGreater(eventsCount.get("jdk.JavaMonitorEnter"), 50);
+        Assert.isGreater(eventsCount.get("jdk.JavaMonitorEnter"), 10);
+        Assert.isGreater(jfrTotalLockDurationMillis / totalLockDurationMillis, 0.80);
         Assert.isGreater(eventsCount.get("jdk.ObjectAllocationInNewTLAB"), 50);
     }
 
@@ -86,7 +103,7 @@ public class JfrTests {
      * @throws Exception Any exception thrown during profiling JFR output parsing.
      */
     @Test(mainClass = JfrMultiModeProfiling.class, agentArgs = "start,all,file=%f.jfr", nameSuffix = "noOverride")
-    @Test(mainClass = JfrMultiModeProfiling.class, agentArgs = "start,all,alloc=100,file=%f.jfr", nameSuffix = "overrideAlloc")
+    @Test(mainClass = JfrMultiModeProfiling.class, agentArgs = "start,all,alloc=262143,file=%f.jfr", nameSuffix = "overrideAlloc")
     public void allModeNoEventOverride(TestProcess p) throws Exception {
         p.waitForExit();
         assert p.exitCode() == 0;
@@ -122,7 +139,7 @@ public class JfrTests {
      * @param p The test process to profile with.
      * @throws Exception Any exception thrown during profiling JFR output parsing.
      */
-    @Test(mainClass = JfrMultiModeProfiling.class, agentArgs = "start,all,event=java.util.Properties.getProperty,alloc=100,file=%f.jfr")
+    @Test(mainClass = JfrMultiModeProfiling.class, agentArgs = "start,all,event=java.util.Properties.getProperty,alloc=262143,file=%f.jfr")
     public void allModeEventOverride(TestProcess p) throws Exception {
         p.waitForExit();
         assert p.exitCode() == 0;
@@ -153,6 +170,26 @@ public class JfrTests {
         assert events.contains("profiler.Free"); // nativemem profiling
     }
 
+    // Simple smoke test, nothing in particular is tested
+    @Test(mainClass = JfrCpuProfiling.class)
+    public void jfrSyncSmoke(TestProcess p) throws Exception {
+        Output out = p.profile("-d 1 --jfrsync default --jfropts 4 -f %f.jfr");
+
+        Set<String> events = new HashSet<>();
+        try (RecordingFile recordingFile = new RecordingFile(p.getFile("%f").toPath())) {
+            while (recordingFile.hasMoreEvents()) {
+                RecordedEvent event = recordingFile.readEvent();
+                events.add(event.getEventType().getName());
+            }
+        }
+
+        assert events.contains("jdk.OSInformation");
+        assert events.contains("jdk.CPUInformation");
+        assert events.contains("jdk.JVMInformation");
+        assert events.contains("jdk.InitialSystemProperty");
+        assert events.contains("jdk.NativeLibrary");
+    }
+
     /**
      * Test to validate time to safepoint profiling
      *
@@ -182,6 +219,14 @@ public class JfrTests {
         assert containsSamplesOutsideWindow(p) : "Expected to find samples outside of ttsp window";
     }
 
+    @Test(mainClass = Hello.class, agentArgs = "start,begin=write,end=write,file=%f.jfr", output = true)
+    public void beginEnd(TestProcess p) throws Exception {
+        Output out = p.waitForExit(TestProcess.STDOUT);
+        assert p.exitCode() == 0;
+
+        assert out.contains("begin and end symbols should not resolve to the same address");
+    }
+
     private boolean containsSamplesOutsideWindow(TestProcess p) throws Exception {
         TreeMap<Instant, Instant> profilerWindows = new TreeMap<>();
         List<RecordedEvent> samples = new ArrayList<>();
