@@ -23,8 +23,8 @@ public class Cpu {
     }
 
     private static void method3() throws Exception {
-        long startTime = System.currentTimeMillis();
-        while (System.currentTimeMillis() - startTime < 10) {
+        long startTime = System.nanoTime();
+        while (System.nanoTime() - startTime < 2_700_000) {
             for (String s : new File("/").list()) {
                 value += s.hashCode();
             }
